@@ -43,9 +43,34 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <cfloat>
 
+/*
+Quality range - compression/subsampling
+100 - lossless RGB compression
+< KIMG_AVIF_QUALITY_BEST, 100 ) - YUV444 color subsampling
+< KIMG_AVIF_QUALITY_HIGH, KIMG_AVIF_QUALITY_BEST ) - YUV422 color subsampling
+< 0, KIMG_AVIF_QUALITY_HIGH ) - YUV420 color subsampling
+< 0, KIMG_AVIF_QUALITY_LOW ) - lossy compression of alpha channel
+*/
+
+#ifndef KIMG_AVIF_DEFAULT_QUALITY
+#define KIMG_AVIF_DEFAULT_QUALITY 68
+#endif
+
+#ifndef KIMG_AVIF_QUALITY_BEST
+#define KIMG_AVIF_QUALITY_BEST 90
+#endif
+
+#ifndef KIMG_AVIF_QUALITY_HIGH
+#define KIMG_AVIF_QUALITY_HIGH 80
+#endif
+
+#ifndef KIMG_AVIF_QUALITY_LOW
+#define KIMG_AVIF_QUALITY_LOW 51
+#endif
+
 QAVIFHandler::QAVIFHandler()
     : m_parseState(ParseAvifNotParsed)
-    , m_quality(52)
+    , m_quality(KIMG_AVIF_DEFAULT_QUALITY)
     , m_container_width(0)
     , m_container_height(0)
     , m_rawAvifData(AVIF_DATA_EMPTY)
@@ -90,7 +115,7 @@ bool QAVIFHandler::canRead(QIODevice *device)
     }
 
     avifROData input;
-    input.data = (const uint8_t *)header.constData();
+    input.data = reinterpret_cast<const uint8_t *>(header.constData());
     input.size = header.size();
 
     if (avifPeekCompatibleFileType(&input)) {
@@ -143,7 +168,7 @@ bool QAVIFHandler::ensureDecoder()
 
     m_rawData = device()->readAll();
 
-    m_rawAvifData.data = (const uint8_t *)m_rawData.constData();
+    m_rawAvifData.data = reinterpret_cast<const uint8_t *>(m_rawData.constData());
     m_rawAvifData.size = m_rawData.size();
 
     if (avifPeekCompatibleFileType(&m_rawAvifData) == AVIF_FALSE) {
@@ -251,11 +276,15 @@ bool QAVIFHandler::decode_one_frame()
     }
 
     bool loadalpha;
+    bool loadgray = false;
 
     if (m_decoder->image->alphaPlane) {
         loadalpha = true;
     } else {
         loadalpha = false;
+        if (m_decoder->image->yuvFormat == AVIF_PIXEL_FORMAT_YUV400) {
+            loadgray = true;
+        }
     }
 
     QImage::Format resultformat;
@@ -288,6 +317,38 @@ bool QAVIFHandler::decode_one_frame()
         if (!colorspace.isValid()) {
             qWarning("AVIF image has Qt-unsupported or invalid ICC profile!");
         }
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 8, 0))
+        else {
+            if (colorspace.colorModel() == QColorSpace::ColorModel::Cmyk) {
+                qWarning("CMYK ICC profile is not expected for AVIF, discarding the ICCprofile!");
+                colorspace = QColorSpace();
+            } else if (colorspace.colorModel() == QColorSpace::ColorModel::Rgb && loadgray) {
+                // Input is GRAY but ICC is RGB, we will return RGB image
+                loadgray = false;
+            } else if (colorspace.colorModel() == QColorSpace::ColorModel::Gray && !loadgray) {
+                // ICC is GRAY but we must return RGB (image has alpha channel for example)
+                // we create similar RGB profile (same whitepoint and TRC)
+                QPointF gray_whitePoint = colorspace.whitePoint();
+                if (gray_whitePoint.isNull()) {
+                    gray_whitePoint = QPointF(0.3127f, 0.329f);
+                }
+
+                const QPointF redP(0.64f, 0.33f);
+                const QPointF greenP(0.3f, 0.6f);
+                const QPointF blueP(0.15f, 0.06f);
+
+                QColorSpace::TransferFunction trc_new = colorspace.transferFunction();
+                float gamma_new = colorspace.gamma();
+                if (trc_new == QColorSpace::TransferFunction::Custom) {
+                    trc_new = QColorSpace::TransferFunction::SRgb;
+                }
+                colorspace = QColorSpace(gray_whitePoint, redP, greenP, blueP, trc_new, gamma_new);
+                if (!colorspace.isValid()) {
+                    qWarning("AVIF plugin created invalid QColorSpace!");
+                }
+            }
+        }
+#endif
     } else {
         float prim[8] = {0.64f, 0.33f, 0.3f, 0.6f, 0.15f, 0.06f, 0.3127f, 0.329f};
         // outPrimaries: rX, rY, gX, gY, bX, bY, wX, wY
@@ -322,6 +383,14 @@ bool QAVIFHandler::decode_one_frame()
         case 13:
             q_trc = QColorSpace::TransferFunction::SRgb;
             break;
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 8, 0))
+        case 16: /* AVIF_TRANSFER_CHARACTERISTICS_PQ */
+            q_trc = QColorSpace::TransferFunction::St2084;
+            break;
+        case 18: /* AVIF_TRANSFER_CHARACTERISTICS_HLG */
+            q_trc = QColorSpace::TransferFunction::Hlg;
+            break;
+#endif
         default:
             qWarning("CICP colorPrimaries: %d, transferCharacteristics: %d\nThe colorspace is unsupported by this plug-in yet.",
                      m_decoder->image->colorPrimaries,
@@ -331,40 +400,50 @@ bool QAVIFHandler::decode_one_frame()
         }
 
         if (q_trc != QColorSpace::TransferFunction::Custom) { // we create new colorspace using Qt
-            switch (m_decoder->image->colorPrimaries) {
-            /* AVIF_COLOR_PRIMARIES_BT709 */
-            case 0:
-            case 1:
-            case 2: /* AVIF_COLOR_PRIMARIES_UNSPECIFIED */
-                colorspace = QColorSpace(QColorSpace::Primaries::SRgb, q_trc, q_trc_gamma);
-                break;
-            /* AVIF_COLOR_PRIMARIES_SMPTE432 */
-            case 12:
-                colorspace = QColorSpace(QColorSpace::Primaries::DciP3D65, q_trc, q_trc_gamma);
-                break;
-            default:
-                colorspace = QColorSpace(whitePoint, redPoint, greenPoint, bluePoint, q_trc, q_trc_gamma);
-                break;
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 8, 0))
+            if (loadgray) {
+                colorspace = QColorSpace(whitePoint, q_trc, q_trc_gamma);
+            } else {
+#endif
+                switch (m_decoder->image->colorPrimaries) {
+                /* AVIF_COLOR_PRIMARIES_BT709 */
+                case 0:
+                case 1:
+                case 2: /* AVIF_COLOR_PRIMARIES_UNSPECIFIED */
+                    colorspace = QColorSpace(QColorSpace::Primaries::SRgb, q_trc, q_trc_gamma);
+                    break;
+                /* AVIF_COLOR_PRIMARIES_SMPTE432 */
+                case 12:
+                    colorspace = QColorSpace(QColorSpace::Primaries::DciP3D65, q_trc, q_trc_gamma);
+                    break;
+                default:
+                    colorspace = QColorSpace(whitePoint, redPoint, greenPoint, bluePoint, q_trc, q_trc_gamma);
+                    break;
+                }
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 8, 0))
             }
+#endif
         }
 
         if (!colorspace.isValid()) {
             qWarning("AVIF plugin created invalid QColorSpace from NCLX/CICP!");
         }
     }
-
-    result.setColorSpace(colorspace);
 #endif
 
     avifRGBImage rgb;
     avifRGBImageSetDefaults(&rgb, m_decoder->image);
 
+#if AVIF_VERSION >= 1000000
+    rgb.maxThreads = m_decoder->maxThreads;
+#endif
+
     if (m_decoder->image->depth > 8) {
         rgb.depth = 16;
         rgb.format = AVIF_RGB_FORMAT_RGBA;
 
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 13, 0))
-        if (!loadalpha && (m_decoder->image->yuvFormat == AVIF_PIXEL_FORMAT_YUV400)) {
+        if (loadgray) {
             resultformat = QImage::Format_Grayscale16;
         }
 #endif
@@ -383,7 +462,7 @@ bool QAVIFHandler::decode_one_frame()
         }
 #endif
 
-        if (!loadalpha && (m_decoder->image->yuvFormat == AVIF_PIXEL_FORMAT_YUV400)) {
+        if (loadgray) {
             resultformat = QImage::Format_Grayscale8;
         }
     }
@@ -455,17 +534,26 @@ bool QAVIFHandler::decode_one_frame()
     }
 
     if (m_decoder->image->transformFlags & AVIF_TRANSFORM_IMIR) {
-#if AVIF_VERSION > 90100
+#if AVIF_VERSION > 90100 && AVIF_VERSION < 1000000
         switch (m_decoder->image->imir.mode) {
 #else
         switch (m_decoder->image->imir.axis) {
 #endif
+#if QT_VERSION < QT_VERSION_CHECK(6, 9, 0)
         case 0: // top-to-bottom
             result = result.mirrored(false, true);
             break;
         case 1: // left-to-right
             result = result.mirrored(true, false);
             break;
+#else
+        case 0: // top-to-bottom
+            result.flip(Qt::Vertical);
+            break;
+        case 1: // left-to-right
+            result.flip(Qt::Horizontal);
+            break;
+#endif
         }
     }
 
@@ -475,6 +563,10 @@ bool QAVIFHandler::decode_one_frame()
         m_current_image = result.convertToFormat(resultformat);
     }
 
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+    m_current_image.setColorSpace(colorspace);
+#endif
+
     m_estimated_dimensions = m_current_image.size();
 
     m_must_jump_to_next_image = false;
@@ -546,9 +638,17 @@ bool QAVIFHandler::write(const QImage &image)
         }
     }
 
+    if (m_quality > 100) {
+        m_quality = 100;
+    } else if (m_quality < 0) {
+        m_quality = KIMG_AVIF_DEFAULT_QUALITY;
+    }
+
+#if AVIF_VERSION < 1000000
     int maxQuantizer = AVIF_QUANTIZER_WORST_QUALITY * (100 - qBound(0, m_quality, 100)) / 100;
     int minQuantizer = 0;
     int maxQuantizerAlpha = 0;
+#endif
     avifResult res;
 
     bool save_grayscale; // true - monochrome, false - colors
@@ -587,6 +687,9 @@ bool QAVIFHandler::write(const QImage &image)
     case QImage::Format_RGBX64:
     case QImage::Format_RGBA64:
     case QImage::Format_RGBA64_Premultiplied:
+#if QT_VERSION >= QT_VERSION_CHECK(6, 8, 0)
+    case QImage::Format_CMYK8888:
+#endif
         save_depth = 10;
         break;
     default:
@@ -598,13 +701,15 @@ bool QAVIFHandler::write(const QImage &image)
         break;
     }
 
-    // quality settings
+#if AVIF_VERSION < 1000000
+    // deprecated quality settings
     if (maxQuantizer > 20) {
         minQuantizer = maxQuantizer - 20;
         if (maxQuantizer > 40) { // we decrease quality of alpha channel here
             maxQuantizerAlpha = maxQuantizer - 40;
         }
     }
+#endif
 
     if (save_grayscale && !image.hasAlphaChannel()) { // we are going to save grayscale image without alpha channel
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 13, 0))
@@ -620,7 +725,15 @@ bool QAVIFHandler::write(const QImage &image)
         QImage tmpgrayimage = image.convertToFormat(tmpformat);
 
         avif = avifImageCreate(tmpgrayimage.width(), tmpgrayimage.height(), save_depth, AVIF_PIXEL_FORMAT_YUV400);
+#if AVIF_VERSION >= 110000
+        res = avifImageAllocatePlanes(avif, AVIF_PLANES_YUV);
+        if (res != AVIF_RESULT_OK) {
+            qWarning("ERROR in avifImageAllocatePlanes: %s", avifResultToString(res));
+            return false;
+        }
+#else
         avifImageAllocatePlanes(avif, AVIF_PLANES_YUV);
+#endif
 
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
         if (tmpgrayimage.colorSpace().isValid()) {
@@ -636,6 +749,14 @@ bool QAVIFHandler::write(const QImage &image)
                 /* AVIF_TRANSFER_CHARACTERISTICS_SRGB */
                 avif->transferCharacteristics = (avifTransferCharacteristics)13;
                 break;
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 8, 0))
+            case QColorSpace::TransferFunction::St2084:
+                avif->transferCharacteristics = (avifTransferCharacteristics)16;
+                break;
+            case QColorSpace::TransferFunction::Hlg:
+                avif->transferCharacteristics = (avifTransferCharacteristics)18;
+                break;
+#endif
             default:
                 /* AVIF_TRANSFER_CHARACTERISTICS_UNSPECIFIED */
                 break;
@@ -681,11 +802,28 @@ bool QAVIFHandler::write(const QImage &image)
             }
         }
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 8, 0)
+        QImage tmpcolorimage;
+        auto cs = image.colorSpace();
+        if (cs.isValid() && cs.colorModel() == QColorSpace::ColorModel::Cmyk && image.format() == QImage::Format_CMYK8888) {
+            tmpcolorimage = image.convertedToColorSpace(QColorSpace(QColorSpace::SRgb), tmpformat);
+        } else if (cs.isValid() && cs.colorModel() == QColorSpace::ColorModel::Gray) {
+            QColorSpace::TransferFunction trc_new = cs.transferFunction();
+            float gamma_new = cs.gamma();
+            if (trc_new == QColorSpace::TransferFunction::Custom) {
+                trc_new = QColorSpace::TransferFunction::SRgb;
+            }
+            tmpcolorimage = image.convertedToColorSpace(QColorSpace(QColorSpace::Primaries::SRgb, trc_new, gamma_new), tmpformat);
+        } else {
+            tmpcolorimage = image.convertToFormat(tmpformat);
+        }
+#else
         QImage tmpcolorimage = image.convertToFormat(tmpformat);
+#endif
 
         avifPixelFormat pixel_format = AVIF_PIXEL_FORMAT_YUV420;
-        if (maxQuantizer < 20) {
-            if (maxQuantizer < 10) {
+        if (m_quality >= KIMG_AVIF_QUALITY_HIGH) {
+            if (m_quality >= KIMG_AVIF_QUALITY_BEST) {
                 pixel_format = AVIF_PIXEL_FORMAT_YUV444; // best quality
             } else {
                 pixel_format = AVIF_PIXEL_FORMAT_YUV422; // high quality
@@ -743,6 +881,14 @@ bool QAVIFHandler::write(const QImage &image)
                 /* AVIF_TRANSFER_CHARACTERISTICS_SRGB */
                 transfer_to_save = (avifTransferCharacteristics)13;
                 break;
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 8, 0))
+            case QColorSpace::TransferFunction::St2084:
+                transfer_to_save = (avifTransferCharacteristics)16;
+                break;
+            case QColorSpace::TransferFunction::Hlg:
+                transfer_to_save = (avifTransferCharacteristics)18;
+                break;
+#endif
             default:
                 /* AVIF_TRANSFER_CHARACTERISTICS_UNSPECIFIED */
                 transfer_to_save = (avifTransferCharacteristics)2;
@@ -758,9 +904,9 @@ bool QAVIFHandler::write(const QImage &image)
                     if (save_depth == 8) {
                         save_depth = 10;
                         if (tmpcolorimage.hasAlphaChannel()) {
-                            tmpcolorimage = tmpcolorimage.convertToFormat(QImage::Format_RGBA64);
+                            tmpcolorimage.convertTo(QImage::Format_RGBA64);
                         } else {
-                            tmpcolorimage = tmpcolorimage.convertToFormat(QImage::Format_RGBX64);
+                            tmpcolorimage.convertTo(QImage::Format_RGBX64);
                         }
                     }
 
@@ -778,6 +924,14 @@ bool QAVIFHandler::write(const QImage &image)
                         case 5: // AVIF_TRANSFER_CHARACTERISTICS_BT470BG
                             tmpcolorimage.convertToColorSpace(QColorSpace(QColorSpace::Primaries::SRgb, 2.8f));
                             break;
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 8, 0))
+                        case 16:
+                            tmpcolorimage.convertToColorSpace(QColorSpace(QColorSpace::Primaries::SRgb, QColorSpace::TransferFunction::St2084));
+                            break;
+                        case 18:
+                            tmpcolorimage.convertToColorSpace(QColorSpace(QColorSpace::Primaries::SRgb, QColorSpace::TransferFunction::Hlg));
+                            break;
+#endif
                         default: // AVIF_TRANSFER_CHARACTERISTICS_SRGB + any other
                             tmpcolorimage.convertToColorSpace(QColorSpace(QColorSpace::Primaries::SRgb, QColorSpace::TransferFunction::SRgb));
                             transfer_to_save = (avifTransferCharacteristics)13;
@@ -812,7 +966,15 @@ bool QAVIFHandler::write(const QImage &image)
         avif->transferCharacteristics = transfer_to_save;
 
         if (iccprofile.size() > 0) {
+#if AVIF_VERSION >= 1000000
+            res = avifImageSetProfileICC(avif, reinterpret_cast<const uint8_t *>(iccprofile.constData()), iccprofile.size());
+            if (res != AVIF_RESULT_OK) {
+                qWarning("ERROR in avifImageSetProfileICC: %s", avifResultToString(res));
+                return false;
+            }
+#else
             avifImageSetProfileICC(avif, reinterpret_cast<const uint8_t *>(iccprofile.constData()), iccprofile.size());
+#endif
         }
 #endif
 
@@ -849,6 +1011,8 @@ bool QAVIFHandler::write(const QImage &image)
     avifRWData raw = AVIF_DATA_EMPTY;
     avifEncoder *encoder = avifEncoderCreate();
     encoder->maxThreads = qBound(1, QThread::idealThreadCount(), 64);
+
+#if AVIF_VERSION < 1000000
     encoder->minQuantizer = minQuantizer;
     encoder->maxQuantizer = maxQuantizer;
 
@@ -856,6 +1020,17 @@ bool QAVIFHandler::write(const QImage &image)
         encoder->minQuantizerAlpha = AVIF_QUANTIZER_LOSSLESS;
         encoder->maxQuantizerAlpha = maxQuantizerAlpha;
     }
+#else
+    encoder->quality = m_quality;
+
+    if (image.hasAlphaChannel()) {
+        if (m_quality >= KIMG_AVIF_QUALITY_LOW) {
+            encoder->qualityAlpha = 100;
+        } else {
+            encoder->qualityAlpha = 100 - (KIMG_AVIF_QUALITY_LOW - m_quality) / 2;
+        }
+    }
+#endif
 
     encoder->speed = 6;
 
@@ -912,7 +1087,7 @@ void QAVIFHandler::setOption(ImageOption option, const QVariant &value)
         if (m_quality > 100) {
             m_quality = 100;
         } else if (m_quality < 0) {
-            m_quality = 52;
+            m_quality = KIMG_AVIF_DEFAULT_QUALITY;
         }
         return;
     default:
@@ -965,6 +1140,8 @@ bool QAVIFHandler::jumpToNextImage()
         return false;
     }
 
+    avifResult decodeResult;
+
     if (m_decoder->imageIndex >= 0) {
         if (m_decoder->imageCount < 2) {
             m_parseState = ParseAvifSuccess;
@@ -972,11 +1149,16 @@ bool QAVIFHandler::jumpToNextImage()
         }
 
         if (m_decoder->imageIndex >= m_decoder->imageCount - 1) { // start from beginning
-            avifDecoderReset(m_decoder);
+            decodeResult = avifDecoderReset(m_decoder);
+            if (decodeResult != AVIF_RESULT_OK) {
+                qWarning("ERROR in avifDecoderReset: %s", avifResultToString(decodeResult));
+                m_parseState = ParseAvifError;
+                return false;
+            }
         }
     }
 
-    avifResult decodeResult = avifDecoderNextImage(m_decoder);
+    decodeResult = avifDecoderNextImage(m_decoder);
 
     if (decodeResult != AVIF_RESULT_OK) {
         qWarning("ERROR: Failed to decode Next image in sequence: %s", avifResultToString(decodeResult));
@@ -1085,6 +1267,11 @@ int QAVIFHandler::loopCount() const
         return 0;
     }
 
+#if AVIF_VERSION >= 1000000
+    if (m_decoder->repetitionCount >= 0) {
+        return m_decoder->repetitionCount;
+    }
+#endif
     // Endless loop
     return -1;
 }
